;=========================================================================
;
;	CRC-32 Calculation, the assembler source was adapted from the 6502
;	code found at http://6502.org/source/integers/crc.htm
;	The generated table has been compared to the table found on 
;	http://www.efg2.com/Lab/Mathematics/CRC.htm and some examples
;	have been checked with the online calculator at
;	http://www.sunshine2k.de/coding/javascript/crc/crc_js.html
;

.equ	crctable3	= 0x1300
.equ	crctable2	= 0x1200
.equ	crctable1	= 0x1100
.equ	crctable0	= 0x1000


crcmaketable:
		lds		temp, a1l
		cpi		temp, 0x42
		brne	crcmaketable010
		rjmp	crcprinttable
crcmaketable010:
		ldi		temp, 0xED
		mov		r7, temp
		ldi		temp, 0xB8
		mov		r6, temp
		ldi		temp, 0x83
		mov		r5, temp
		ldi		temp, 0x20
		mov		r4, temp
		clr		xl
crcbyteloop:
		mov		crcl, xl
		clr		crch
		clr		crc3
		clr		crc4
		ldi		count, 8
crcbitloop:
		lsr		crc4
		ror		crc3
		ror		crch
		ror		crcl
		brcc	crcnoadd
		eor		crc4, r7
		eor		crc3, r6
		eor		crch, r5
		eor		crcl, r4
crcnoadd:
		dec		count
		brne	crcbitloop
		ldi		xh, high(crctable3)
		st		X, crc4
		dec		xh
		st		X, crc3
		dec		xh
		st		X, crch
		dec		xh
		st		X, crcl
		inc		xl
		brne	crcbyteloop
		ret
;
;	aaaa.bbbbY
;
crcupdate:
		mov		crcl, ff
		mov		crch, ff
		mov		crc3, ff
		mov		crc4, ff
crcnext:
		lds		xl, a1l
		lds		xh, a1h
		call	load
;	EOR crcl         ; Quick CRC computation with lookup tables
;	TAX
;	LDA crch
;	EOR CRCT0,X
;	STA crc
;	LDA crc3
;	EOR CRCT1,X
;	STA crch
;	LDA crc4
;	EOR CRCT2,X
;	STA crc3
;	LDA CRCT3,X
;	STA crc4
;	RTS
		eor		r0, crcl				;	EOR	crcl
		mov		zl, r0					;	TAX
		ldi		zh, high(crctable0)		;	LDA	CRCT0, X
		ld		r0, Z	
		eor		r0, crch				;	EOR	crch
		mov		crcl, r0				;	STA	crc

		inc		zh						;	LDA	CRCT1, X
		ld		r0, Z
		eor		r0, crc3				;	EOR	crc3
		mov		crch, r0				;	STA	crch
		
		inc		zh						;	LDA	CRCT2, X
		ld		r0, Z
		eor		r0, crc4				;	EOR	crc4
		mov		crc3, r0				;	STA	crc3
		
		inc		zh						;	LDA	CRCT3, X
		ld		crc4, Z					;	STA	crc4
		rcall	nxta1
		brlo	crcnext


		eor		crcl, ff
		eor		crch, ff
		eor		crc3, ff
		eor		crc4, ff

		sts		pprint+0, crcl
		sts		pprint+1, crch
		sts		pprint+2, crc3
		sts		pprint+3, crc4

		call	print
		.db		"CRC standard is  0x", 0x83, 0x82, 0x81, 0x80, CR, LF, 0

		ret


;
;	Cycle optimized CRC calculation testing the byte swapping
;
crcupdate2:
	mov		crcl, ff
	mov		crch, ff
	mov		crc3, ff
	mov		crc4, ff
;	Initially	->	(CRC4, CRC3, CRCH, CRCL)

crcnext20:
	lds		xl, a1l					;;;
	lds		xh, a1h					;;;
	call	load					;;;
	mov		zl, r0					;;;

	eor		zl, crcl				;  1	EOR	crcl
	ldi		zh, high(crctable0)		;  1	LDA	CRCT0, X
	ld		r0, Z					;  2
	eor		crch, r0				;  1	EOR	crch

	inc		zh						;  1	LDA	CRCT1, X
	ld		r0, Z					;  2
	eor		crc3, r0				;  1	EOR	crc3

	inc		zh						;  1	LDA	CRCT2, X
	ld		r0, Z					;  2
	eor		crc4, r0				;  1	EOR	crc4

	inc		zh						;  1	LDA	CRCT3, X
	ld		crcl, Z					;  2	STA	crc4
	
	rcall	nxta1
	brlo	crcnext21

;	1st Byte	->	(CRCL, CRC4, CRC3, CRCH)

	eor		crcl, ff
	eor		crch, ff
	eor		crc3, ff
	eor		crc4, ff
	sts		pprint+0, crch
	sts		pprint+1, crc3
	sts		pprint+2, crc4
	sts		pprint+3, crcl
	rjmp	crcnext2print
	
crcnext21:
	lds		xl, a1l					;;;
	lds		xh, a1h					;;;
	call	load					;;;
	mov		zl, r0					;;;

	eor		zl, crch				;  1	EOR	crcl
	ldi		zh, high(crctable0)		;  1	LDA	CRCT0, X
	ld		r0, Z					;  2
	eor		crc3, r0				;  1	EOR	crch

	inc		zh						;  1	LDA	CRCT1, X
	ld		r0, Z					;  2
	eor		crc4, r0				;  1	EOR	crc3

	inc		zh						;  1	LDA	CRCT2, X
	ld		r0, Z					;  2
	eor		crcl, r0				;  1	EOR	crc4

	inc		zh						;  1	LDA	CRCT3, X
	ld		crch, Z					;  2	STA	crc4

	rcall	nxta1
	brlo	crcnext22

;	2nd Byte	->	(CRCH, CRCL, CRC4, CRC3)
	eor		crcl, ff
	eor		crch, ff
	eor		crc3, ff
	eor		crc4, ff
	sts		pprint+0, crc3
	sts		pprint+1, crc4
	sts		pprint+2, crcl
	sts		pprint+3, crch
	rjmp	crcnext2print

crcnext22:
	lds		xl, a1l					;;;
	lds		xh, a1h					;;;
	call	load					;;;
	mov		zl, r0					;;;

	eor		zl, crc3				;  1	EOR	crcl
	ldi		zh, high(crctable0)		;  1	LDA	CRCT0, X
	ld		r0, Z					;  2
	eor		crc4, r0				;  1	EOR	crch

	inc		zh						;  1	LDA	CRCT1, X
	ld		r0, Z					;  2
	eor		crcl, r0				;  1	EOR	crc3

	inc		zh						;  1	LDA	CRCT2, X
	ld		r0, Z					;  2
	eor		crch, r0				;  1	EOR	crc4

	inc		zh						;  1	LDA	CRCT3, X
	ld		crc3, Z					;  2	STA	crc4

	rcall	nxta1
	brlo	crcnext23

;	3rd	Byte	->	(CRC3, CRCH, CRCL, CRC4)
	eor		crcl, ff
	eor		crch, ff
	eor		crc3, ff
	eor		crc4, ff
	sts		pprint+0, crc4
	sts		pprint+1, crcl
	sts		pprint+2, crch
	sts		pprint+3, crc3
	rjmp	crcnext2print

crcnext23:
	lds		xl, a1l					;;;
	lds		xh, a1h					;;;
	call	load					;;;
	mov		zl, r0					;;;

	eor		zl, crc4				;  1	EOR	crcl
	ldi		zh, high(crctable0)		;  1	LDA	CRCT0, X
	ld		r0, Z					;  2
	eor		crcl, r0				;  1	EOR	crch

	inc		zh						;  1	LDA	CRCT1, X
	ld		r0, Z					;  2
	eor		crch, r0				;  1	EOR	crc3

	inc		zh						;  1	LDA	CRCT2, X
	ld		r0, Z					;  2
	eor		crc3, r0				;  1	EOR	crc4

	inc		zh						;  1	LDA	CRCT3, X
	ld		crc4, Z					;  2	STA	crc4

	rcall	nxta1
	brlo	crcnext24

;	4th Byte	->	(CRC4, CRC3, CRCH, CRCL)
	eor		crcl, ff
	eor		crch, ff
	eor		crc3, ff
	eor		crc4, ff
	sts		pprint+0, crcl
	sts		pprint+1, crch
	sts		pprint+2, crc3
	sts		pprint+3, crc4
	rjmp	crcnext2print

crcnext24:
	rjmp	crcnext20

crcnext2print:
	call	print
	.db		"CRC optimized is 0x", 0x83, 0x82, 0x81, 0x80, CR, LF, 0
	ret


crcprintline:
		call	print
		.db		0x08, "0x", 0x80, ", 0x", 0x81, ", 0x", 0x82, ", 0x", 0x83, ", 0x", 0x84, ", 0x", 0x85, ", 0x", 0x86, ", 0x", 0x87, ", 0x", 0x88, ", 0x", 0x89, ", 0x", 0x8a, ", 0x", 0x8b, ", 0x", 0x8c, ", 0x", 0x8d, ", 0x", 0x8e, ", 0x", 0x8f, CR, LF, 0
		ret

crcprinttable:
		ldi		xl, low(crctable0)
		ldi		xh, high(crctable0)
		call	print
		.db		"crctable0: ", CR, LF, 0
		rcall	crcprintsub
		call	print
		.db		"crctable1: ", CR, LF, 0
		rcall	crcprintsub
		call	print
		.db		"crctable2: ", CR, LF, 0
		rcall	crcprintsub
		call	print
		.db		"crctable3: ", CR, LF, 0
		rcall	crcprintsub
		ret
		
crcprintsub:
		ldi		count, 16
crcprinttable010:
		ldi		yl, low(pprint)
		ldi		yh, high(pprint)
		ldi		temp, 16
crcprinttable020:		
		ld		char, X+
		st		Y+, char
		dec		temp
		brne	crcprinttable020
		rcall	crcprintline
		dec		count
		brne	crcprinttable010
		ret
		
		
#ifdef test
;
;	For a MFM Harddisk emulator we need to send address and
;	data records and we need to be able to receive data records
;	when the controller writes a sector, especially when sending
;	we need to generate the CRC/ECC. In theory we have enough
;	time to calculate it on the fly. We use zl directly as the
;	data register as we also need it as the low-byte of the
;	index into the CRC tables
;
	sbic	b_RQST					;  2
	rjmp	PC_1					;  2
	ld		zl, X+					;  2
	out		PORTA, zl				;  1
	ack								;  2
									; ==
									;  9

;
;	The following code can be optimized in at least
;	two ways
;	-	Instead of storing the table in flash we
;		could create or keep a copy in RAM, the total
;		table is 1kbyte. Using ld instead of lpm saves
;		4 cycles
;	-	instead of transferring the new CRC into the
;		correct CRC regisgters (CRCL, CRCH, CRC3, CRC4)
;		we could swap the registers, that is after each
;		CRC calculation the registers would be rotated
;
;		Initially	->	(CRC4, CRC3, CRCH, CRCL)
;		1st Byte	->	(CRCL, CRC4, CRC3, CRCH)
;		2nd Byte	->	(CRCH, CRCL, CRC4, CRC3)
;		3rd	Byte	->	(CRC3, CRCH, CRCL, CRC4)
;		4th Byte	->	(CRC4, CRC3, CRCH, CRCL)
;
;		for this the eor r0, crcx must be swapped eor crcx,r0
;		and the following mov can be ommitted. After 4 CRC
;		calculations the 32-bit CRC is again stored in the
;		natural order in the CRC registers. This saves another
;		3 cycles
;
	eor		zl, crcl				;  1	EOR	crcl
	ldi		zh, high(2*crctable0)	;  1	LDA	CRCT0, X
	lpm		r0, Z					;  3
	eor		r0, crch				;  1	EOR	crch
	mov		crcl, r0				;  1	STA	crc

	inc		zh						;  1	LDA	CRCT1, X
	lpm		r0, Z					;  3
	eor		r0, crc3				;  1	EOR	crc3
	mov		crch, r0				;  1	STA	crch

	inc		zh						;  1	LDA	CRCT2, X
	lpm		r0, Z					;  3
	eor		r0, crc4				;  1	EOR	crc4
	mov		crc3, r0				;  1	STA	crc3

	inc		zh						;  1	LDA	CRCT3, X
	lpm		crc4, Z					;  3	STA	crc4
									; ==
									; 23
									; -4	ld instead lpm
									; -3	register swapping
									; ==

									; 16
;	
;	At the data rate of 5Mbps of MFM harddisks we have 1.6usec
;	for each byte. When running the MCU at 20MHz this corresponds
;	to 32 cycles. The optimized code uses 25 cycles. See above
;	test at "crcupdate2"
;


;
;
;

	sbic	b_RQST					;  2
	rjmp	PC_1					;  2
	eor		crcl, ff				;  1
	out		PORTA, crcl				;  1
									; ==
									;  6

	sbic	b_RQST					;  2
	rjmp	PC_1					;  2
	eor		crch, ff				;  1
	out		PORTA, crch				;  1
									; ==
									;  6

	sbic	b_RQST					;  2
	rjmp	PC_1					;  2
	eor		crc3, ff				;  1
	out		PORTA, crc3				;  1
									; ==
									;  6

	sbic	b_RQST					;  2
	rjmp	PC_1					;  2
	eor		crc4, ff				;  1
	out		PORTA, crc4				;  1
									; ==
									;  6

;
;	Assume we only use a 16MHz processor then we only have 25 cycles
;	
	ld		zl, X+					;  3
	sbic	d_DRQ					;  2
	rjmp	PC-1
	sts		WD, zl					;  3
	sbic							;  2

	eor		zl, crcl				;  1	EOR	crcl
	ldi		zh, high(crctable0)		;  1	LDA	CRCT0, X
	ld		r0, Z					;  2
	eor		crch, r0				;  1	EOR	crch

	inc		zh						;  1	LDA	CRCT1, X
	ld		r0, Z					;  2
	eor		crc3, r0				;  1	EOR	crc3

	inc		zh						;  1	LDA	CRCT2, X
	ld		r0, Z					;  2
	eor		crc4, r0				;  1	EOR	crc4

	inc		zh						;  1	LDA	CRCT3, X
	ld		crcl, Z					;  2	STA	crc4
									; --
									; 24

;
;	This is to tight. 
#endif
