.include	"m1284Pdef.inc"
.listmac


.def	ff		= r9
.def	one		= r10
.def	zero 	= r11
.def	temp 	= r16				; Temporay register with immediate always destroyed!
.def	char	= r17
.def	count	= r18

#define clibuffer InputBuffer		; use this buffer for monitor
#define charout   serout
#define charin    serin


.equ	F_CPU		= 18432000		; Crystal clock

.equ	BAUD0		= 115200		; Baudrate USART0
.equ	BAUD1		= 115200		; Baudrate USART1

.equ	CR		=	0x0d		; carriage return
.equ	LF		=	0x0a		; line feed
.equ	ESC		=	0x1b		; ESC to exit

.equ	t_long	= int(3.5*F_CPU/1000000)
.equ	t_short	= int(2.5*F_CPU/1000000)

;--------------------------------------------------------------------------
;
; Baud Rate Calculator
;
.equ	UBRR0_VAL   = ((F_CPU+BAUD0*8)/(BAUD0*16)-1)	; nearest value
.equ	BAUD0_REAL  = (F_CPU/(16*(UBRR0_VAL+1)))		; effective baudrate
.equ	BAUD0_ERROR = ((BAUD0_REAL*1000)/BAUD0-1000)	; error in 0/00
.if ((BAUD0_ERROR>10) || (BAUD0_ERROR<-10))
	.error " USART0: Effective baudrate is more then 1 percent off requested baudrate"
.endif
.equ	UBRR1_VAL   = ((F_CPU+BAUD1*8)/(BAUD1*16)-1)	; nearest value
.equ	BAUD1_REAL  = (F_CPU/(16*(UBRR1_VAL+1)))		; effective baudrate
.equ	BAUD1_ERROR = ((BAUD1_REAL*1000)/BAUD1-1000)	; error in 0/00
.if ((BAUD1_ERROR>10) || (BAUD1_ERROR<-10))
	.error " USART1: Effective baudrate is more then 1 percent off requested baudrate"
.endif



;--------------------------------------------------------------------------
;
;	Data
;
	.dseg
InputBuffer:
	.byte	80
track:
	.byte	0
;--------------------------------------------------------------------------
;
;	Flash
;
	.cseg
.org	0
	jmp		reset
;.org	INT0addr
;	rjmp	INT0_isr
;.org	INT1addr
;	rjmp	INT1_isr
;.org	INT2addr
;	rjmp	INT2_isr
;.org	PCI2addr
;	rjmp	PC_INIT
;.org	WDTaddr
;	rjmp	wdt_isr
;.org	OC0Aaddr
;	rjmp	OC0_isr
;.org	URXC0addr
;	rjmp	rxc0_isr
;.org	UDRE0addr
;	rjmp	dre0_isr
	
.org	INT_VECTORS_SIZE

	.dw	t_long
	.dw	t_short


;--------------------------------------------------------------------------
;
;	Power Up and Watchdog Reset
;
reset:
;
;	Set Stack Pointer to top of RAM
;
	ldi		temp,low(RAMEND) 
	out		SPL,temp 
	ldi		temp,high(RAMEND) 
	out		SPH,temp
;
;	Init Constants
;
	clr		zero
	clr		one
	inc		one
	clr		ff
	dec		ff

;--------------------------------------------------------------------------
;
;	USART0
;	
;	Interactive user interface to control the AVR
;
;	115200baud, 8 data, no parity, 1 stop bit
;
	ldi		temp, HIGH(UBRR0_VAL)
	sts		UBRR0H, temp
	ldi		temp, LOW(UBRR0_VAL)
	sts		UBRR0L, temp
;
;	Enable receiver and transmitter and receiver interrupt
;
	ldi		temp, (1<<RXEN0)|(1<<TXEN0)|(1<<RXCIE0)
	sts		UCSR0B, temp
;
;	Set frame format: 8data, 1stop bit, no parity
;
	ldi		temp, (3<<UCSZ00)
	sts		UCSR0C, temp

;--------------------------------------------------------------------------
;
;	Port A - Data Port
;
	out		DDRA, zero				; Initially configured as input

;--------------------------------------------------------------------------
;
;	Port B - Unused
;
	out		DDRB, zero

;--------------------------------------------------------------------------
;
;		Port C
;
;
;		Port C
;
.equ	DC	=	7		; Disk Change		Input
.equ	HS	=	6		; Head Select		Output
.equ	RD	=	5		; Read Data			Input
.equ	WP	=	4		; Write Protect		Input
.equ	TRK0=	3		; Track 0			Input
.equ	WG	=	2		; Write Gate		Output
.equ	WD	=	1		; Write Data		Output
.equ	STEP=	0		; Step				Output

	out		PORTC, ff
	ldi		temp, (1<<HS) | (1<<WG) | (1<<WD) | (1<<STEP)
	out		DDRC, temp

;--------------------------------------------------------------------------
;
;		Port D
;
.equ	DIR	=	7		; Direction			Output
.equ	MOT	=	6		; Motor On			Output
.equ	DS	=	5		; Drive Select		Output
.equ	IDX	=	4		; Index Pulse		Input

	out		PORTD, ff
	ldi		temp, (1<<DIR) | (1<<MOT) | (1<<DS)
	out		DDRD, temp
;--------------------------------------------------------------------------
;
;		Done
;
	rcall	splashscreen
	rjmp	mon


;--------------------------------------------------------------------------
;
;		Serial 
;
serout:
	push	char
seroutwait:
	lds		char, UCSR0A
	sbrs	char, UDRE0
	rjmp	seroutwait
	pop		char
	sts		UDR0,char
	ret			
serin:
	wdr
	lds		char, UCSR0A
	sbrs	char, RXC0
	rjmp	serin
	lds		char, UDR0
	ret

;--------------------------------------------------------------------------
;
;	Splashscreen
;
splashscreen:
	ldi		zl, low(2*hello)
	ldi		zh, high(2*hello)

splashsend:
	lpm		char ,Z+ 
	tst		char
	breq	splashdone
	rcall	serout
	rjmp	splashsend

splashdone:
	ret

;--------------------------------------------------------------------------
;
;	AVR Monitor, inlcude the default commands followed by application
;	specific commands
;
.include	"../include/monitor-chartbl-v1-3.asm"
	.db		0, '?'
	.db		0, 'O'		
	.db		0, 'S'
	.db		0, 'T'		;		Seek
	.db		0, 'R'
	.db		0, 'R' & 0x1f
	
;
;	AVR Monitor, inlcude the action routine address table of the default
;	commands followed by the addresses of the application specific 
;	command routines. Both tables must be in sync of course...
;
.include	"../include/monitor-subtbl-v1-3.asm"
	.dw		helpscreen
	.dw		togglemot
	.dw		toggleds
	.dw		seek
	.dw		readtrack
	.dw		readtrack2


togglemot:
	ldi		temp, (1<<MOT)
	out		PIND, temp
	ret

toggleds:
	ldi		temp, (1<<DS)
	out		PIND, temp
	ret
	
delay100usec:
	push	count
	clr		count				; 1
delay100loop:
	rjmp	delaynext			;  2
delaynext:
	rjmp	PC+1				;  2
	rjmp	PC+1				;  2
	rjmp	PC+1				;  2
	rjmp	PC+1				;  2
								; 10
	rjmp	PC+1				;  2
	rjmp	PC+1				;  2
	rjmp	PC+1				;  2
	rjmp	PC+1				;  2
	rjmp	PC+1				;  2
								; 20
	rjmp	PC+1				;  2
	rjmp	PC+1				;  2
	rjmp	PC+1				;  2
	rjmp	PC+1				;  2
	rjmp	PC+1				;  2
								; 30
	rjmp	PC+1				;  2
	rjmp	PC+1				;  2
	rjmp	PC+1				;  2
	rjmp	PC+1				;  2
	rjmp	PC+1				;  2
								; 40
	rjmp	PC+1				;  2
	rjmp	PC+1				;  2
	rjmp	PC+1				;  2
	rjmp	PC+1				;  2
	rjmp	PC+1				;  2
								; 50
	rjmp	PC+1				;  2
	rjmp	PC+1				;  2
	rjmp	PC+1				;  2
	rjmp	PC+1				;  2
	rjmp	PC+1				;  2
								; 60
	rjmp	PC+1				;  2
	rjmp	PC+1				;  2
	rjmp	PC+1				;  2
	rjmp	PC+1				;  2
	rjmp	PC+1				;  2
								; 70
	dec		count
	brne	delay100loop
	pop		count
	dec		count
	brne	delay100usec
	ret
	
	
seek:
	cbi		PORTD, MOT			; Motor On
	cbi		PORTD, DS
	ldi		count, 0
	rcall	delay100usec		; Wait 25.6ms
	lds		temp, a1l
	tst		temp				; Track 00
	breq	seekhome
	cpi		temp, 80
	brlo	seekok
	rjmp	seekdone
seekok:
	lds		r0, track			; r0 = current track
	sts		track, temp			; temp = desired track
	cp		temp, r0
	brlo	seekout
	brne	seekin
	rjmp	seekdone
seekin:
	cbi		PORTD, DIR			; Low = towards centre
	ldi		count, 10
	rcall	delay100usec
	cbi		PORTC, STEP
	rjmp	PC+1				; Pulse must have 0.8usec
	rjmp	PC+1				; At least 16 cycles @ 20MHz
	rjmp	PC+1
	rjmp	PC+1
	rjmp	PC+1
	rjmp	PC+1
	rjmp	PC+1
	rjmp	PC+1
	rjmp	PC+1
	rjmp	PC+1
	sbi		PORTC, STEP	
	ldi		count, 30
	rcall	delay100usec
	inc		temp
	cp		temp, r0
	brne	seekin
	rjmp	seekdone

seekout:
	sbi		PORTD, DIR			; High = away from centre
	ldi		count, 10
	rcall	delay100usec
	cbi		PORTC, STEP
	rjmp	PC+1				; Pulse must have 0.8usec
	rjmp	PC+1				; At least 16 cycles @ 20MHz
	rjmp	PC+1
	rjmp	PC+1
	rjmp	PC+1
	rjmp	PC+1
	rjmp	PC+1
	rjmp	PC+1
	rjmp	PC+1
	rjmp	PC+1
	sbi		PORTC, STEP	
	ldi		count, 30
	rcall	delay100usec
	dec		temp
	cp		temp, r0
	brne	seekout
	rjmp	seekdone

seekhome:
	sbi		PORTD, DIR
	ldi		count, 10
	rcall	delay100usec
	cbi		PORTC, STEP
	rjmp	PC+1				; Pulse must have 0.8usec
	rjmp	PC+1				; At least 16 cycles @ 20MHz
	rjmp	PC+1
	rjmp	PC+1
	rjmp	PC+1
	rjmp	PC+1
	rjmp	PC+1
	rjmp	PC+1
	rjmp	PC+1
	rjmp	PC+1
	sbi		PORTC, STEP	
	ldi		count, 30
	rcall	delay100usec
	sbic	PINC, TRK0
	rjmp	seekhome
	sts		track, zero

seekdone:
	ldi		count, 100
	rcall	delay100usec
	sbi		PORTD, MOT
	sbi		PORTD, DS
	ret
	
;--------------------------------------------------------------------------
;
;
readtrack:
	cbi		PORTD, MOT
	cbi		PORTD, DS
	clr		count
	rcall	delay100usec
	
	ldi		yl, low(0x1000)
	ldi		yh, high(0x1000)
	
;	ldi		temp, byte1(0x8000)
;	mov		r4, temp
;	ldi		temp, byte2(0x8000)
;	mov		r5, temp
;	ldi		temp, byte3(0x8000)
;	mov		r6, temp

	movw	r5:r4, yh:yl
		
	sts		TCCR2A, zero
	sts		TCCR2B, one
	sts		TCNT2, zero

;    register unsigned char counter;
;    long totalBits=0;
;    long target = ((long)RAW_TRACKDATA_LENGTH)*(long)8;
;
;    while (totalBits<target) {
;        for (register unsigned char bits=0; bits<4; bits++) {
;            // Wait while pin is high
;            
;            while (PIN_READ_DATA_PORT & PIN_READ_DATA_MASK) {};
;            counter = TCNT2, TCNT2 = 0;  // reset - must be done with a COMMA
;           
;            DataOutputByte<<=2;   
;
;            // DO NOT USE BRACES HERE, use the "," or the optomiser messes it up
;            if (counter<80) DataOutputByte|=B00000001,totalBits+=2; else    // this accounts for just a '1' or a '01' as two '1' arent allowed in a row
;            if (counter>111) DataOutputByte|=B00000011,totalBits+=4; else DataOutputByte|=B00000010,totalBits+=3;
;            
;            // Wait until pin is high again
;            while (!(PIN_READ_DATA_PORT & PIN_READ_DATA_MASK)) {};
;        }
;        UDR0 = DataOutputByte;
;    }
readtrack000:
	clr		xl
	ldi		count, 4
readtrack010:
	sbis	PINC, RD
	rjmp	PC-1
	sbic	PINC, RD
	rjmp	PC-1				; Wait until low
	lds		temp, TCNT2
	sts		TCNT2, zero
	lsl		char
	lsl		char
	cpi		temp, 46
	brlo	readtrack020
	cpi		temp, 64
	brlo	readtrack030
	ori		char, 3
	subi	xl, -4
	rjmp	readtrack040
readtrack030:
	ori		char, 2
	subi	xl, -3
	rjmp	readtrack040
readtrack020:
	ori		char, 1
	subi	xl, -2
readtrack040:
	dec		count
	brne	readtrack010
	st		Y+, char

	sub		r4, one
	sbc		r5, zero
	brne	readtrack000

;	sub		r4, xl
;	sbc		r5, zero
;	sbc		r6, zero
;	brpl	readtrack000
	sbi		PORTD, MOT
	sbi		PORTD, DS
	ret	
;--------------------------------------------------------------------------
;
;
;

;.equ	tracksize	= 12544			; Multiple of 256 and larger than 12500bytes net track size
.equ	tracksize	= 0x2000		; test
.equ	trackbuffer	= 0x1000		; where we place the buffer during tests
.equ	syncmarks	= 0xF00			; where we place found sync marks

	.macro	btime
	sbis	PINC, RD
	rjmp	PC-1
	sbic	PINC, RD
	rjmp	PC-1
	lds		temp, TCNT2
	sts		TCNT2, zero
	.endmacro

readtrack2:
	cbi		PORTD, MOT
	cbi		PORTD, DS
	clr		count
	rcall	delay100usec
;
;	Initialise
;
	ldi		xl, low(tracksize)		; Bytes to collect
	ldi		xh, high(tracksize)
	ldi		yl, low(trackbuffer)	; Buffer
	ldi		yh, high(trackbuffer)

	ldi		temp, 0xa5
readfill:
	st		Y+, temp
	sbiw	xh:xl, 1
	brne	readfill
;;;	rjmp	readexit

	ldi		xl, low(tracksize)		; Bytes to collect
	ldi		xh, high(tracksize)
	ldi		yl, low(trackbuffer)	; Buffer
	ldi		yh, high(trackbuffer)

	ldi		zl, low(syncmarks)		; List of SYNC Marks
	ldi		zh, high(syncmarks)
	sts		TCCR2A, zero
	sts		TCCR2B, one
	sts		TCNT2, zero

sstart:
	ldi		count, 80				; less than 12*8
sloop:
	btime
	cpi		temp, t_short
	brsh	sstart
	dec		count
	brne	sloop
;
;	Note typically there are 3 consecutive SYNC MARKS
;	LMLMSLMLMSLMLM
;
wlong1:
	btime					; L
	cpi		temp, t_long
	brlo	wlong1
;
	btime					; M
	cpi		temp, t_short
	brlo	wlong1
	cpi		temp, t_long
	brsh	wlong1
;
	btime					; L
	cpi		temp, t_long
	brlo	wlong1
;
	btime					; M
	cpi		temp, t_short
	brlo	wlong1
	cpi		temp, t_long
	brsh	wlong1
;
	btime					; S
	cpi		temp, t_short
	brlo	wlong2
wlong11:
	rjmp	sstart
;	
wlong2:
	btime					; L
	cpi		temp, t_long
	brlo	wlong11
;
	btime					; M
	cpi		temp, t_short
	brlo	wlong11
	cpi		temp, t_long
	brsh	wlong11
;
	btime					; L
	cpi		temp, t_long
	brlo	wlong11
;
	btime					; M
	cpi		temp, t_short
	brlo	wlong11
	cpi		temp, t_long
	brsh	wlong11
;
	btime					; S
	cpi		temp, t_short
	brlo	wlong3
wlong22:
	rjmp	sstart
;
wlong3:
	btime					; L
	cpi		temp, t_long
	brlo	wlong22
;
	btime					; M
	cpi		temp, t_short
	brlo	wlong22
	cpi		temp, t_long
	brsh	wlong22
;
	btime					; L
	cpi		temp, t_long
	brlo	wlong22
;
	btime					; M
	cpi		temp, t_short
	brlo	wlong22
	cpi		temp, t_long
	brsh	wlong22
	
;
;	we spotted  a sync mark now we need to analyze further gaps
;	there are several cases depending on the data following the
;	sync mark.
;
l0000:
	ldi		count, 8
l0010:															; Worst case must be less than 37 cycles (short gap)
	btime														;  8
	cpi		temp, t_short										;  1
	brsh	l0020												;  2	-> 11
;---------------------------------------------------------
;
;	Next Bit is a '1'
;	
;	Data	1 0 1 0 0 0 0 1| 1
;	Clock	 0 0 0 1 1 1 0 |0
;	MFM		100010010001001|01
;
	sec															;
	rol		char												;
	dec		count												;
	brne	l0010												;
	st		Y+, char											;
	ldi		count, 8											;
	sbiw	xh:xl, 1											;
	brne	l0010												;
	rjmp	readtrackdone										;

l0020:
	cpi		temp, t_long										;  1
	brlo	l0040												;  2	-> 14
;---------------------------------------------------------
;
;	Next are two bits '01'
;	
;	Data	1 0 1 0 0 0 0 1| 0 1
;	Clock	 0 0 0 1 1 1 0 |0 0
;	MFM		100010010001001|0001
;
	lsl		char												;
	dec		count												;
	brne	l0030												;
	st		Y+, char											;
	ldi		count, 8											;
	sbiw	xh:xl, 1											;
	brne	l0030												;
	rjmp	readtrackdone										;
l0030:
	sec															;
	rol		char												;
	dec		count												;
	brne	l0010												;
	st		Y+, char											;
	ldi		count, 8											;
	sbiw	xh:xl, 1											;
	brne	l0010												;
	rjmp	readtrackdone
;---------------------------------------------------------
;
;	1 1/2 bit cells so we have a clock pulse therefore
;	next two bits are '00'
;			------------------->
;	Data	1 0 1 0 0 0 0 1| 0 0
;	Clock	 0 0 0 1 1 1 0 |0 1 
;	MFM		100010010001001|001
;
l0040:
	lsl		char												;  1
	dec		count												;  1
	brne	l0050												;  2	-> 18
	st		Y+, char											;
	ldi		count, 8											;
	sbiw	xh:xl, 1											;
	brne	l0050												;
	rjmp	readtrackdone
;
l0050:
	lsl		char												;  1
	dec		count												;  1
	brne	l0060												;  1
	st		Y+, char											;  2
	ldi		count, 8											;  1
	sbiw	xh:xl, 1											;  2
	brne	l0060												;  2	-> 28
	rjmp	readtrackdone
;---------------------------------------------------------
;
;	pulse was on a clock and the next data bit is either
;	a zero or a one in which case the next gap is either
;	a short (zero) or a medium(one) a long gap at this
;	stage would be equivalent to a sync mark
;
l0060:
	btime
	cpi		temp, t_short
	brlo	l0080
	cpi		temp, t_long
	brsh	l0090
;---------------------------------------------------------
;
;	we have a medium gap and therefore the data bit is one
;	and we are again aligned to the data bits
;
;			------------------->
;	Data	1 0 1 0 0 0 0 1| 0 0 1
;	Clock	 0 0 0 1 1 1 0 |0 1 0
;	MFM		100010010001001|001001
;
	sec
	rol		char
	dec		count
	brne	l0070
	st		Y+, char
	ldi		count, 8
	sbiw	xh:xl, 1
	breq	readtrackdone
l0070:
	rjmp	l0010
;---------------------------------------------------------
;
;	we have a short pulse, and therefore the data bit is
;	zero and we are back at the situation where the pulse
;	was on a clock
;
;			------------------->
;	Data	1 0 1 0 0 0 0 1| 0 0 0
;	Clock	 0 0 0 1 1 1 0 |0 1 1
;	MFM		100010010001001|00101
;
l0080:
	lsl		char
	dec		count
	brne	l0060
	st		Y+, char
	ldi		count, 8
	sbiw	xh:xl, 1
	brne	l0060
;---------------------------------------------------------
;
;	we have a long gap, at this stage this is equivalent
;	to a missing clock pulse, a long gap with missing
;	clock pulse means a medium gap and then again
;	data is expected
;
;			------------------->
;	Data	1 0 1 0 0 0 0 1| 0 0 0 0
;	Clock	 0 0 0 1 1 1 0 |0 1 0 1
;	MFM		100010010001001|0010001
;
l0090:
	st		Z+, yl
	st		Z+, yh
;	rjmp	l0000
;
;
readtrackdone:

	rjmp	readexit
readexit:
	sbi		PORTD, MOT
	sbi		PORTD, DS
	ret	

;--------------------------------------------------------------------------
;
;	Floppy Emu CPLD 
;
;	if (_wreq == 0) begin
;		// was there a transition on the wr line?
;		if (wrHistory[1] && ~wrHistory[0]) begin

;	ldi		bitCounter, 16
;readtrack110:
;	sbic	PINC, RD
;	rjmp	PC-1				; Make sure RD = Low
;	sts		TCNT2, zero			; Init counter
;readtrack120:
;	sbis	PINC, RD
;	rjmp	PC-1
;	sbic	PINC, RD
;	rjmp	PC-1				; At next high->low transition		
;	lds		bitTimer, TCNT2		; retrieve counter value
;	cpi		bitTimer, 9
;	brlo	readtrack110
;	cpi		bitTimer, 27
;	brlo	readtrack150
	
;	lsl		shifter
;	ldi		temp, 15
;	sts		TCNT2, temp
;	dec		bitCounter
;	brne	readtrack120
;	rjmp	readtrack160
;readtrack150:
;	sec
;	rol		shifter
;	dec		bitCounter
;	brne	readtrack120
;readtrack160:
;	rol		shifterh
	
;	cpi		shifter, 0x44
;	cpi		shifterh, 0x89
	
;			// has at least half a bit cell time elpased since the last cell boundary?
;			if (bitTimer >= 10 &&) begin 
;				shifter <= { shifter[6:0], 1'b1 }; 
;				bitCounter <= bitCounter - 1'b1;					
;			end
;			// do nothing if the clock count was less than half a cell
;			// reset the bit timer
;			bitTimer <= 0;
;		end
;		else begin
;			// have one and a half bit cell times elapsed?
;			if (bitTimer >= 30) begin
;				shifter <= { shifter[6:0], 1'b0 };
;				bitCounter <= bitCounter - 1'b1;
;				bitTimer <= 10;
;			end
;			else begin
;				// init shifter at the beginning of a write, so we can recognize the framing bits later
;				if (wrClear == 0) begin
;					shifter <= 0;
;					wrClear <= 1;
;				end
;				// has a complete byte been shifted in?
;				// If we're in write mode, but haven't yet synched (framed the bytes in the bit stream),
;				// and we see 01000100 in the shifter, assume that it's the first half of an A1 sync
;				if ((bitCounter == 0) ||
;					 (shifter == 8'h44 && mfmWriteSynced == 0)) begin		
;					// MFM: send the mcu the data nibble in the low 4 bits, and clock bit C2 in bit 4
;					wrData[0] <= shifter[0];
;					wrData[1] <= shifter[2];
;					wrData[2] <= shifter[4];
;					wrData[3] <= shifter[6];
;					wrData[4] <= shifter[5]; // clock bit
;					bitCounter <= 8;
;					// signal the mcu that a new nibble is ready
;					rdAckWrTick <= (shifter == 8'h44 && mfmWriteSynced == 0) ? 0 : ~rdAckWrTick; 
;					mfmWriteSynced <= mfmWriteSynced | (shifter == 8'h44);
;				end
;				bitTimer <= bitTimer + 1'b1;	
;			end	
;		end
;	end

	sbis	PINC, RD
	rjmp	PC-1
	sbic	PINC, RD
	rjmp	PC-1				


;--------------------------------------------------------------------------
;
;	AVR monitor. A interactive monitor inspired by the Apple II
;
.include	"../include/monitor-v1-6.asm"


;--------------------------------------------------------------------------
;
;	Monitor Function Routines
;
helpscreen:
	lds		temp, a1l
	cpi		temp, 0x01
	brne	helpscreen2
	ldi		zl, low(2*help1)
	ldi		zh, high(2*help1)
	jmp		splashsend
helpscreen2:
	cpi		temp, 0x02
	brne	helpscreen3
	ldi		zl, low(2*help2)
	ldi		zh, high(2*help2)
	jmp		splashsend
helpscreen3:
	cpi		temp, 0x03
	brne	helpscreen4
	ldi		zl, low(2*help3)
	ldi		zh, high(2*help3)
	jmp		splashsend
helpscreen4:
	cpi		temp, 0x04
	brne	helpscreen5
	ldi		zl, low(2*help4)
	ldi		zh, high(2*help4)
	jmp		splashsend
helpscreen5:						; Default Help Screen
	ldi		zl, low(2*hello)
	ldi		zh, high(2*hello)
	jmp		splashsend

hello:	.db	0x0d, 0x0a, 0x0d, 0x0a
;			"----+----1----+----2----+----3----+----4----+----5----+----6----+----7----+----8"
		.db	"MFM Read Test Environment Vers 0.1  ",__DATE__,"                               ",  0x0d, 0x0a
		.db	"                                                                                ", 0x0d, 0x0a
		.db	"     For general syntax see Apple II monitor. The following commands of the     ", 0x0d, 0x0a
		.db	"     Apple II Monitor are implemented: blank, dot, colon, -, +, <, move, verify ", 0x0d, 0x0a
		.db	"     ?        Show Helpscreen                                                   ", 0x0d, 0x0a
		.db	"                                                                                ", 0x0d, 0x0a
		.db	"                                                                                ", 0x0d, 0x0a
		.db	"                                                                                ", 0x0d, 0x0a
		.db	"                                                                                ", 0x0d, 0x0a
		.db	"                                                                                ", 0x0d, 0x0a
		.db	"                                                                                ", 0x0d, 0x0a
		.db	"                                                                                ", 0x0d, 0x0a
		.db	"                                                                                ", 0x0d, 0x0a
		.db	"                                                                                ", 0x0d, 0x0a
		.db	"                                                                                ", 0x0d, 0x0a
		.db	"                                                                                ", 0x0d, 0x0a
		.db	"                                                                                ", 0x0d, 0x0a
		.dw	0
;
help1:	.db	0x0d, 0x0a, 0x0d, 0x0a
;			"----+----1----+----2----+----3----+----4----+----5----+----6----+----7----+----8"
		.db	"Qbridge w/ SD-Card Support Ver. 0.1 ",__DATE__," Help Screen 1                 ",  0x0d, 0x0a
		.db	"                                                                                ", 0x0d, 0x0a
		.db	"     The normal sequence to initialise normal SD card is                        ", 0x0d, 0x0a
		.db	"     I        Set SPI Mode (CMD0)                                               ", 0x0d, 0x0a
		.db	"     H        Set Interface Condition (CMD8)                                    ", 0x0d, 0x0a
		.db	"              Version 1 Card do not understand this command and return an error ", 0x0d, 0x0a
		.db	"              IFC:05FFFFFFFF                                                    ", 0x0d, 0x0a
		.db	"              Version 2 Card do understand this command and return success      ", 0x0d, 0x0a
		.db	"              IFC:01000001AA                                                    ", 0x0d, 0x0a
		.db	"     J        Then initialises the card (ACMD41)                                ", 0x0d, 0x0a
		.db	"     K        SD Card Operation Code Registers (CMD58)                          ", 0x0d, 0x0a
		.db	"              The operation code register will detect SD or SDHC cards          ", 0x0d, 0x0a
		.db	"              R58:0080FF8000FF,HCXC:00  Normal SD Cards                         ", 0x0d, 0x0a
		.db	"              R58:00C0FF8000FF,HCXC:40  SDHC Cards                              ", 0x0d, 0x0a
		.db	"     L        SD Card Set Block Length to 512bytes (CMD16)                      ", 0x0d, 0x0a
		.db	"              Only non-HC/XC SD Cards require to set the block size             ", 0x0d, 0x0a
		.db	"              HC/XC Cards have the block size already set to 512bytes           ", 0x0d, 0x0a
		.db	"                                                                                ", 0x0d, 0x0a
		.db	"     ^R       xxxx<yyyy^R Read Sector yyyy to Address xxxx                      ", 0x0d, 0x0a
		.db	"     ^W       xxxx<yyyy^W Write Address yyyy to Sector xxxx                     ", 0x0d, 0x0a
		.db	"                                                                                ", 0x0d, 0x0a
		.dw	0
;
help2:	.db	0x0d, 0x0a, 0x0d, 0x0a
;			"----+----1----+----2----+----3----+----4----+----5----+----6----+----7----+----8"
		.db	"Qbridge w/ SD-Card Support Ver. 0.1 ",__DATE__," Help Screen 2                 ",  0x0d, 0x0a
		.db	"                                                                                ", 0x0d, 0x0a
		.db	"     Q        Mount volume and show info                                        ", 0x0d, 0x0a
 		.db	"     T        Show subdirectory starting at cluster (0 for root dir)            ", 0x0d, 0x0a
 		.db	"     G        Show fragment list of file with start cluster nnnn                ", 0x0d, 0x0a
		.db	"     N        Enter Command Line Mode                                           ", 0x0d, 0x0a
;		.db	"     U        Find Name in Directory                                            ", 0x0d, 0x0a
		.db	"     Y        aaaa<bbbbR Read Virtual Sector bbbb to address aaaa               ", 0x0d, 0x0a
;		.db	"     Z        Find Full File Name                                               ", 0x0d, 0x0a
		.db	"                                                                                ", 0x0d, 0x0a
		.dw	0
;
help3:	.db	0x0d, 0x0a, 0x0d, 0x0a
;			"----+----1----+----2----+----3----+----4----+----5----+----6----+----7----+----8"
		.db	"Qbridge w/ SD-Card Support Ver. 0.1 ",__DATE__," Help Screen 3                 ",  0x0d, 0x0a
		.db	"                                                                                ", 0x0d, 0x0a
		.db	"     CD <directorypath>                                                         ", 0x0d, 0x0a
		.db	"              Change working directory to the given path, an empty string       ", 0x0d, 0x0a
		.db	"              sets the working directory to the root (empty)                    ", 0x0d, 0x0a
		.db	"                                                                                ", 0x0d, 0x0a
		.db	"     DIR <directorypath>                                                        ", 0x0d, 0x0a
		.db	"              Display the contents of the directory, when no path is            ", 0x0d, 0x0a
		.db	"              specified the current directory is listed                         ", 0x0d, 0x0a
		.db	"                                                                                ", 0x0d, 0x0a
		.db	"     PWD                                                                        ", 0x0d, 0x0a
		.db	"              Print Working Directory, that is the current base path            ", 0x0d, 0x0a
		.db	"                                                                                ", 0x0d, 0x0a
		.db	"     TYPE <filename>                                                            ", 0x0d, 0x0a
		.db	"              Prints the content of the first 64kbyte of the file to the        ", 0x0d, 0x0a
		.db	"              terminal. File lookup starts from the current directory.  The     ", 0x0d, 0x0a
		.db	"              Filename may inlcude sub-directories                              ", 0x0d, 0x0a
		.db	"                                                                                ", 0x0d, 0x0a
		.db	"     ATTACH <filename> TO DLn: (work in progress)                               ", 0x0d, 0x0a
		.db	"              Attach a file as diskimage to the DL volume n:                    ", 0x0d, 0x0a
		.db	"                                                                                ", 0x0d, 0x0a
		.db	"                                                                                ", 0x0d, 0x0a
		.dw	0
;
help4:	.db	0x0d, 0x0a, 0x0d, 0x0a
;			"----+----1----+----2----+----3----+----4----+----5----+----6----+----7----+----8"
		.db	"Qbridge w/ SD-Card Support Ver. 0.1 ",__DATE__," Help Screen 4                 ",  0x0d, 0x0a
		.db	"                                                                                ", 0x0d, 0x0a
		.db	"     R        aaaa<bbbb.ccccR DMA read from PDP-11 Memory bbbb to AVR aaaa      ", 0x0d, 0x0a
		.db	"     W        aaaa<bbbb.ccccW DMA write from AVR Memory bbbb to PDP-11 aaaa     ", 0x0d, 0x0a
		.db	"     O        Increment DMA address                                             ", 0x0d, 0x0a		
		.db	"     n$       CRDY: n=0 clear CRDY, n=1 set CRDY, else show CRDY                ", 0x0d, 0x0a		
		.db	"     ^I       Generate vectored interrupt to PDP-11                             ", 0x0d, 0x0a
		.db	"     ^W       Write 0..65535 to PDP-11 memory on 64kbyte boundary and check     ", 0x0d, 0x0a
		.db	"     ^L       Initialise Log Buffer and Reset CSRs                              ", 0x0d, 0x0a
		.db	"     ^Z       Show current status, port, counters, flags                        ", 0x0d, 0x0a
		.dw	0























