;--------------------------------------------------------------------------
;
;	MFM Writer
;
;	1. Write Sector
;
;	When writing a sector we need to write the complete data segment
;
;	+--------+----+----+----------------------------+-----+---------+
;	|preamble|sync|DAM |             Data           | CRC |postamble|
;	|  12 x  | 3x | 1x |            512 x           | 2x  |   40 x  |
;	|  0x00  |0xA1|0xFB|            user            | CRC |   0x4E  |
;	+--------+----+----+----------------------------+-----+---------+
;
;	First we need to find the correct sector using the "readsection"
;	routine. 
;
;

write_nextidsegment:
	ldi		xl, low(7)				; Bytes to collect
	ldi		xh, high(7)
	ldi		yl, low(idsegment)
	ldi		yh, high(idsegment)
	clr		miss
	sbi		PORTB, 1				; Debugging output
	rcall	readsection
	
	brcs	write_readiderr
;
;	Now we have quite some time. Because after the ID segment there
;	is a 22 x 0x4E postamble. This is equivalent to 352usec se we
;	have about 7000 clock cycles
;
;	We first setup the timer which counts the ticks after readsection
;	returns so we know when to start writing
;
	ldi
	out		TCNT3L
	ldi
	out		TCNT3H
;
;	And do now the "house-keeping"
;
	mov		crcl, ff
	mov		crch, ff
	ldi		temp, 0xA1
	updcrc	temp
	ldi		temp, 0xA1
	updcrc	temp
	ldi		temp, 0xA1
	updcrc	temp
	lds		temp, idsegment
	cpi		temp, 0xFE
	brne	write_nextidsegment		; It's not a idsegment
	updcrc	temp
	lds		temp, idsegment+1		; Track
	updcrc	temp
	lds		temp, idsegment+2		; Side
	updcrc	temp
	lds		temp, idsegment+3		; Sector
	updcrc	temp
	lds		temp, idsegment+4		; Length
	updcrc	temp
	lds		temp, idsegment+5
	cp		crcl, temp
	lds		temp, idsegment+6
	cpc		crch, temp
	brne	write_readiderr
	lds		temp, idsegment+3
	lds		r0, writesector
	cp		temp, r0
	brne	write_nextidsegment
	
write_waitgap:
	lds		temp, TCCR3A
	sbrs	temp, TOV3
	rjmp	write_waitgap
;
;	WD is connected to MOSI
;	The MCU must be clocked at a multiple of 4MHz
;	Each Byte is converted into four patterns being
;	sent over SPI to the Floppy. 
;
;	The pattern depends on the last bit sent
;	the two bits we want to sent and the next bit
;	we will be sending. So we have a total of 16
;	pattern
;
;			------------------>
;	Data	1 0 1 0 0 0 0 1| 0 0 0
;	Clock	 0 0 0 1 1 1 0 |0 1 1
;	MFM		100010010001001|00101


;	Data	0 |0 0 |0 
;	Clock	 1| 1 1| 1
;	MFM		01|0101|01		Pattern is 11011101

;	Data	0 |0 1 |0 
;	Clock	 1| 0 0| 1
;	MFM		01|0010|01		Pattern is 11110111

;	Data	0 |1 0 |0 
;	Clock	 0| 0 1| 1
;	MFM		00|1001|01		Pattern is 01111101

;	Data	0 |1 1 |0 
;	Clock	 0| 0 0| 1
;	MFM		00|1010|01		Pattern is 01110111


;	Data	0 |0 0 |1 
;	Clock	 1| 1 0| 0
;	MFM		01|0100|10		Pattern is 11011111

;	Data	0 |0 1 |1 
;	Clock	 1| 0 0| 0
;	MFM		01|0010|10		Pattern is 11110111

;	Data	0 |1 0 |1 
;	Clock	 0| 0 0| 0
;	MFM		00|1000|11		Pattern is 01111111

;	Data	0 |1 1 |1 
;	Clock	 0| 0 0| 0
;	MFM		00|1010|10		Pattern is 01110111


;	Data	1 |0 0 |0 
;	Clock	 0| 1 1| 1
;	MFM		10|0101|01		Pattern is 11011101

;	Data	1 |0 1 |0 
;	Clock	 0| 0 0| 1
;	MFM		10|0010|01		Pattern is 11110111

;	Data	1 |1 0 |0 
;	Clock	 0| 0 1| 1
;	MFM		10|1001|01		Pattern is 01111101

;	Data	1 |1 1 |0 
;	Clock	 0| 0 0| 1
;	MFM		10|1010|01		Pattern is 01110111


;	Data	1 |0 0 |1 
;	Clock	 0| 1 0| 0
;	MFM		10|0100|10		Pattern is 11011111

;	Data	1 |0 1 |1 
;	Clock	 0| 0 0| 0
;	MFM		10|0010|10		Pattern is 11110111

;	Data	1 |1 0 |1 
;	Clock	 0| 0 0| 0
;	MFM		10|1000|11		Pattern is 01111111

;	Data	1 |1 1 |1 
;	Clock	 0| 0 0| 0
;	MFM		10|1010|10		Pattern is 01110111

;	So we need create a window of 4 bits into our
;	data we want to send convert the window into
;	a pattern and then shift the windows 2 bits
;	to the right SPI is clocked with 4MHz 
;	and the MCU with 16MHz so the algorithm is
;	allowed to use 16/4*8 = 32 cycles only


	ldi		temp, 
	cbi		WG
	nop
	nop
	sts		SPD, temp
	nop
	nop
	nop
	nop
	sts		SPD, temp
	
;